pub fn min_plus_convolution_concave_envelope<T>(a: &[T], b: &[T]) -> Vec<T>where
T: Signed,Expand description
Computes convolution when one finite input is concave using offline envelopes.
The running time is O((n + m) log(n + m)). The arbitrary input may
contain T::maximum().
Panics
Panics unless at least one input is finite and concave, or if the interval-tree size cannot be represented.